#
# asmgen/makefile  (for GNU)
#
# This software was written by Leonard Janke (janke@unixg.ubc.Sa)
# in 1996-7 and is entered, by him, into the public domain.

include ../config.gnu

ASMFILES=md5cp.S sha0cp.S sha1cp.S rmd128cp.S rmd160cp.S
TARGETDIR=../chnhash
EXECUTABLES=cmd5$(EXESUFFIX) csha0$(EXESUFFIX) csha1$(EXESUFFIX) \
            crmd128$(EXESUFFIX) crmd160$(EXESUFFIX)

CLINKHDR=clink.h

all:	$(ASMFILES)

#
# assembly generation helpers
#

a2r160.o:	a2r160.cpp a2r160.hpp
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

a2r128.o:	a2r128.cpp a2r128.hpp
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

#
# abstract base classes
#

asmgen.o:	asmgen.cpp  asmgen.hpp
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

shag.o:	shag.cpp asmgen.hpp a2r160.hpp shag.hpp 
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE)  $<

#
# gen stuff 
#

cmd5deps=a2r128.o asmgen.o md5g.o cmd5.o 

cmd5$(EXESUFFIX):	$(cmd5deps)  
	$(LINK) $(EXENAME) $@ $(cmd5deps)

cmd5.o:	cmd5.cpp asmgen.hpp md5g.hpp
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

csha0deps=a2r160.o asmgen.o shag.o sha0g.o csha0.o 

csha0$(EXESUFFIX):	$(csha0deps)  
	$(LINK) $(EXENAME) $@  $(csha0deps)

csha0.o:	csha0.cpp asmgen.hpp shag.hpp sha0g.hpp
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

csha1deps=a2r160.o asmgen.o shag.o sha1g.o csha1.o 

csha1$(EXESUFFIX):	$(csha1deps)  
	$(LINK) $(EXENAME) $@ $(csha1deps)

csha1.o:	csha1.cpp asmgen.hpp shag.hpp sha1g.hpp 
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

crmd128deps=a2r128.o asmgen.o rmd128g.o crmd128.o 

crmd128$(EXESUFFIX):	$(crmd128deps)  
	$(LINK) $(EXENAME) $@ $(crmd128deps)

crmd128.o:	crmd128.cpp asmgen.hpp rmd128g.hpp
	$(CPP) $(CPPFLAGS) $(CPPCOMPILE) $<

crmd160deps=a2r160.o asmgen.o rmd160g.o crmd160.o 

crmd160$(EXESUFFIX):	$(crmd160deps)  
	$(LINK) $(EXENAME) $@ $(crmd160deps)

crmd160.o:	 crmd160.cpp asmgen.hpp rmd160g.hpp
	$(CPP) $(CPPFLAGS) $(ASMCALLCONV) $(CPPCOMPILE) $<

#
# MD5 stuff
#

md5cp.S:	cmd5$(EXESUFFIX) 
	./cmd5$(EXESUFFIX) $(ASMFORMAT) > md5cp.S

md5g.o:	md5g.cpp asmgen.hpp a2r128.hpp md5g.hpp
	$(CPP) $(CPPFLAGS) $(ASMCALLCONV) $(CPPCOMPILE) $<

#
# SHA stuff
#

# SHA1

sha1cp.S:	csha1$(EXESUFFIX)
	./csha1$(EXESUFFIX) $(ASMFORMAT) > sha1cp.S

sha1g.o:	 sha1g.cpp asmgen.hpp a2r160.hpp shag.hpp sha1g.hpp
	$(CPP) $(CPPFLAGS) $(ASMCALLCONV) $(CPPCOMPILE) $<

# SHA0

sha0cp.S:	csha0$(EXESUFFIX) 
	./csha0$(EXESUFFIX) $(ASMFORMAT) > sha0cp.S

sha0g.o:	sha0g.cpp asmgen.hpp a2r160.hpp shag.hpp sha0g.hpp
	$(CPP) $(CPPFLAGS) $(ASMCALLCONV) $(CPPCOMPILE) $<

#
# RIPEMD-128 stuff
#

rmd128cp.S: 	crmd128$(EXESUFFIX)  
	./crmd128$(EXESUFFIX) $(ASMFORMAT) > rmd128cp.S

rmd128g.o:	rmd128g.cpp asmgen.hpp a2r128.hpp rmd128g.hpp
	$(CPP) $(CPPFLAGS) $(ASMCALLCONV) $(CPPCOMPILE) $<

#
# RIPEMD-160 stuff
#

rmd160cp.S: 	crmd160$(EXESUFFIX) 
	./crmd160$(EXESUFFIX) $(ASMFORMAT) > rmd160cp.S

rmd160g.o:	rmd160g.cpp asmgen.hpp a2r160.hpp rmd160g.hpp
	$(CPP) $(CPPFLAGS) $(ASMCALLCONV) $(CPPCOMPILE) $<

#
# installation
#

install:	$(ASMFILES)
	cp *cp.S $(TARGETDIR) ; cp $(CLINKHDR) $(TARGETDIR)/clink.h

#
# clean up 
#

clean:
	rm -f *~ *.o $(EXECUTABLES)

distclean:	clean 
	rm -f *.S
