#
# asmgen/makefile 
# create assembly files required for libch Version 0.11w
#
# This software was written by Leonard Janke (janke@unixg.ubc.ca)
# in 1996-7 and is entered, by him, into the public domain.

!include ..\config.ms

CFILES=md5cp.c sha0cp.c sha1cp.c rmd128cp.c rmd160cp.c
TARGETDIR=..\chnhash

all : $(CFILES)

#
# assembly generation helpers
#

a2r160.obj :	a2r160.hpp a2r160.cpp
	$(CPP) $(CPPQUIET) $*.cpp

a2r128.obj :	a2r128.hpp a2r128.cpp
	$(CPP) $(CPPQUIET) $*.cpp

#
# abstract base classes
#

asmgen.obj :	asmgen.hpp  asmgen.cpp
	$(CPP) $(CPPQUIET) $*.cpp

shag.obj :	asmgen.hpp  a2r160.hpp shag.hpp shag.cpp
	$(CPP) $(CPPQUIET) $*.cpp

#
# MD5 stuff
#

md5cp.c :	cmd5.exe 
	cmd5.exe masm-inl-cdecl > md5cp.c

md5g.obj :	asmgen.hpp  a2r128.hpp md5g.hpp md5g.cpp
	$(CPP) $(CPPQUIET) $*.cpp

cmd5deps=a2r128.obj asmgen.obj md5g.obj cmd5.obj 
cmd5link=a2r128.obj asmgen.obj md5g.obj cmd5.obj 

cmd5.exe :	$(cmd5deps)  
	$(LINKER) $(LINKQUIET) $(LINKFILE) $(cmd5link) $(EXENAME)$@

cmd5.obj :	asmgen.hpp  md5g.hpp cmd5.cpp
	$(CPP) $(CPPQUIET) $*.cpp

#
# SHA stuff
#

# SHA1

sha1cp.c :	csha1.exe 
	csha1.exe masm-inl-cdecl > sha1cp.c

sha1g.obj :	asmgen.hpp  a2r160.hpp shag.hpp sha1g.hpp sha1g.cpp
	$(CPP) $(CPPQUIET) $*.cpp

csha1deps=csha1.obj shag.obj sha1g.obj asmgen.obj a2r160.obj
csha1link=csha1.obj shag.obj sha1g.obj asmgen.obj a2r160.obj

csha1.exe :	$(csha1deps)
	$(LINKER) $(LINKQUIET) $(LINKFILE)  $(csha1link) $(EXENAME)$@

csha1.obj :	asmgen.hpp  shag.hpp sha1g.hpp csha1.cpp
	$(CPP) $(CPPQUIET) $*.cpp

# SHA0

sha0cp.c :	csha0.exe 
	csha0.exe masm-inl-cdecl > sha0cp.c

sha0g.obj :	asmgen.hpp  a2r160.hpp shag.hpp sha0g.hpp sha0g.cpp
	$(CPP) $(CPPQUIET) $*.cpp

csha0deps=csha0.obj shag.obj sha0g.obj asmgen.obj a2r160.obj
csha0link=csha0.obj shag.obj sha0g.obj asmgen.obj a2r160.obj

csha0.exe :	$(csha0deps)
	$(LINKER) $(LINKQUIET) $(LINKFILE) $(csha0link) $(EXENAME)$@

csha0.obj :	asmgen.hpp  shag.hpp sha0g.hpp csha0.cpp
	$(CPP) $(CPPQUIET) $*.cpp

#
# RIPE-MD128 stuff
#

rmd128cp.c : 	crmd128.exe  
	crmd128.exe masm-inl-cdecl > rmd128cp.c

rmd128g.obj :	asmgen.hpp  a2r128.hpp rmd128g.hpp rmd128g.cpp
	$(CPP) $(CPPQUIET) $*.cpp

crmd128deps=crmd128.obj rmd128g.obj asmgen.obj a2r128.obj
crmd128link=crmd128.obj rmd128g.obj asmgen.obj a2r128.obj

crmd128.exe :	$(crmd128deps)
	$(LINKER) $(LINKQUIET) $(LINKFILE) $(crmd128link) $(EXENAME)$@

crmd128.obj :	asmgen.hpp  rmd128g.hpp crmd128.cpp
	$(CPP) $(CPPQUIET) $*.cpp

#
# RIPE-MD160 stuff
#

rmd160cp.c : 	crmd160.exe  
	crmd160.exe masm-inl-cdecl > rmd160cp.c

rmd160g.obj :	asmgen.hpp a2r160.hpp rmd160g.hpp rmd160g.cpp
	$(CPP) $(CPPQUIET) $*.cpp

crmd160deps=crmd160.obj rmd160g.obj asmgen.obj a2r160.obj
crmd160link=crmd160.obj rmd160g.obj asmgen.obj a2r160.obj

crmd160.exe :	$(crmd160deps)
	$(LINKER) $(LINKQUIET) $(LINKFILE) $(crmd160link) $(EXENAME)$@

crmd160.obj :	asmgen.hpp  rmd160g.hpp crmd160.cpp
	$(CPP) $(CPPQUIET) $*.cpp

#
# installation
#

install :	$(CFILES)
	$(COPY) *cp.c $(TARGETDIR)
	$(COPY) clink.h $(TARGETDIR)\clink.h

#
# clean up 
#

clean :
	if exist *~ $(DELETE) *~ 
	if exist *.obj $(DELETE) *.obj 
	if exist *.exe $(DELETE) *.exe 

distclean :	clean 
	if exist *cp.c $(DELETE) *cp.c 
