#
# asmgen/makefile 
#
# This software was written by Leonard Janke (janke@unixg.ubc.asma)
# in 1996-7 and is entered, by him, into the public domain.

!include ..\config.wat

ASMFILES=md5cp.asm sha0cp.asm sha1cp.asm rmd128cp.asm rmd160cp.asm
TARGETDIR=..\chnhash

CLINKHDR=clink.h

all :	$(ASMFILES) .SYMBOLIC

#
# assembly generation helpers
#

a2r160.obj :	a2r160.hpp a2r160.cpp
	$(CPP) $(CPPQUIET) $].

a2r128.obj :	a2r128.hpp a2r128.cpp
	$(CPP) $(CPPQUIET) $].

#
# abstract base classes
#

asmgen.obj :	asmgen.hpp  asmgen.cpp
	$(CPP) $(CPPQUIET) $].

shag.obj :	asmgen.hpp  a2r160.hpp shag.hpp shag.cpp
	$(CPP) $(CPPQUIET) $].

#
# gen stuff 
#

cmd5deps=a2r128.obj asmgen.obj md5g.obj cmd5.obj 
cmd5link=a2r128.obj,asmgen.obj,md5g.obj,cmd5.obj 

cmd5.exe :	$(cmd5deps)  
	$(LINK) $(LINKQUIET) $(LINKFILE) $(cmd5link) $(EXENAME) $^.

cmd5.obj :	asmgen.hpp md5g.hpp cmd5.cpp
	$(CPP) $(CPPQUIET) $].

csha0deps=a2r160.obj asmgen.obj shag.obj sha0g.obj csha0.obj 
csha0link=a2r160.obj,asmgen.obj,shag.obj,sha0g.obj,csha0.obj 

csha0.exe :	$(csha0deps)  
	$(LINK) $(LINKQUIET) $(LINKFILE) $(csha0link) $(EXENAME) $^.

csha0.obj :	asmgen.hpp shag.hpp sha0g.hpp csha0.cpp
	$(CPP) $(CPPQUIET) $].

csha1deps=a2r160.obj asmgen.obj shag.obj sha1g.obj csha1.obj 
csha1link=a2r160.obj,asmgen.obj,shag.obj,sha1g.obj,csha1.obj 

csha1.exe :	$(csha1deps)  
	$(LINK) $(LINKQUIET) $(LINKFILE) $(csha1link) $(EXENAME) $^.

csha1.obj :	asmgen.hpp shag.hpp sha1g.hpp csha1.cpp
	$(CPP) $(CPPQUIET) $].

crmd128deps=a2r128.obj asmgen.obj rmd128g.obj crmd128.obj 
crmd128link=a2r128.obj,asmgen.obj,rmd128g.obj,crmd128.obj 

crmd128.exe :	$(crmd128deps)  
	$(LINK) $(LINKQUIET) $(LINKFILE) $(crmd128link) $(EXENAME) $^.

crmd128.obj :	asmgen.hpp rmd128g.hpp crmd128.cpp
	$(CPP) $(CPPQUIET) $].

crmd160deps=a2r160.obj asmgen.obj rmd160g.obj crmd160.obj 
crmd160link=a2r160.obj,asmgen.obj,rmd160g.obj,crmd160.obj 

crmd160.exe :	$(crmd160deps)  
	$(LINK) $(LINKQUIET) $(LINKFILE) $(crmd160link) $(EXENAME) $^.

crmd160.obj :	asmgen.hpp rmd160g.hpp crmd160.cpp
	$(CPP) $(CPPQUIET) $].

#
# MD5 stuff
#

md5cp.asm :	cmd5.exe 
	cmd5.exe $(ASMFORMAT) > md5cp.asm

md5g.obj :	asmgen.hpp  a2r128.hpp md5g.hpp md5g.cpp
	$(CPP) $(CPPQUIET) $].

#
# SHA stuff
#

# SHA1

sha1cp.asm :	csha1.exe 
	csha1.exe $(ASMFORMAT) > sha1cp.asm

sha1g.obj :	asmgen.hpp  a2r160.hpp shag.hpp sha1g.hpp sha1g.cpp
	$(CPP) $(CPPQUIET) $].

# SHA0

sha0cp.asm :	csha0.exe 
	csha0.exe $(ASMFORMAT) > sha0cp.asm

sha0g.obj :	asmgen.hpp  a2r160.hpp shag.hpp sha0g.hpp sha0g.cpp
	$(CPP) $(CPPQUIET) $].

#
# RIPEMD-128 stuff
#

rmd128cp.asm : 	crmd128.exe  
	crmd128.exe $(ASMFORMAT) > rmd128cp.asm

rmd128g.obj :	asmgen.hpp  a2r128.hpp rmd128g.hpp rmd128g.cpp
	$(CPP) $(CPPQUIET) $].

#
# RIPEMD-160 stuff
#

rmd160cp.asm : 	crmd160.exe  
	crmd160.exe $(ASMFORMAT) > rmd160cp.asm

rmd160g.obj :	asmgen.hpp  a2r160.hpp rmd160g.hpp rmd160g.cpp
	$(CPP) $(CPPQUIET) $].

#
# installation
#

install :	$(ASMFILES) .SYMBOLIC
	$(COPY) *cp.asm $(TARGETDIR)
	$(COPY) $(CLINKHDR) $(TARGETDIR)\clink.h

#
# clean up 
#

clean
	@if exist *~ $(DELETE) *~ 
	@if exist *.obj $(DELETE) *.obj 
	@if exist *.exe $(DELETE) *.exe 
	@if exist *.err $(DELETE) *.err 

distclean :	clean  .SYMBOLIC
	@if exist *.asm $(DELETE) *.asm
