#
# chnhash/makefile.gnu 
#
# creates a C++ wrapper to Pentium optimized assembly functions
# for (possibly) one-way hashing
#
# This softare was written by Leonard Janke (janke@unixg.ubc.ca)
# in 1996-7 and is entered, by him, into the public domain.

include ../config.gnu

OBJS=chnhash.o sha.o sha0.o sha1.o md4ish.o md5.o rmd128.o rmd160.o \
     md5cp.o sha0cp.o sha1cp.o rmd128cp.o rmd160cp.o
INCLUDE=-I../
LIBDIR=../lib
NAME=ch
MISCHEADERS=chmisc.hpp chmisc.inl chtypes.h clink.h

lib$(NAME).a:	$(OBJS)
	$(LIB) lib$(NAME).a $? 

# compression functions

%.o:	%.S 
	$(ASSEMBLER) $(ASMOUT) $@ $<

#
# abstract base classes
#

chnhash.o:	chnhash.cpp chnhash.hpp
	$(CPP) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

md4ish.o:	md4ish.cpp $(MISCHEADERS) chnhash.hpp md4ish.hpp
	$(CPP) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

#
# MD5 stuff
#

md5.o:	md5.cpp $(MISCHEADERS) chnhash.hpp md4ish.hpp md5.hpp 
	$(CPP) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

#
# SHA stuff
#

sha.o:	sha.cpp $(MISCHEADERS) chnhash.hpp sha.hpp
	$(CPP) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

sha1.o:	sha1.cpp $(MISCHEADERS) chnhash.hpp sha.hpp sha1.hpp
	$(CPP) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

sha0.o:	sha0.cpp $(MISCHEADERS) chnhash.hpp sha.hpp sha0.hpp
	$(CPP) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

#
# RIPE-MD128 stuff
#

rmd128.o:	rmd128.cpp $(MISCHEADERS) chnhash.hpp rmd128.hpp
	$(CPP) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

#
# RIPE-MD160 stuff
#

rmd160.o:	rmd160.cpp $(MISCHEADERS) chnhash.hpp rmd160.hpp
	$(CPP) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDE) $(CPPCOMPILE) $<

#
# installation stuff
#

install-lib:	lib$(NAME).a 
	cp lib$(NAME).a $(LIBDIR)/lib$(NAME).a 

install-headers:
	cp *.hpp *.h *.inl ../include/chnhash/

install:	install-lib install-headers

#
# clean up stuff
#

clean:
	rm -f *~ *.o *.a

distclean:	clean
	rm -f *.S clink.h
