#
# chnhash/makefile.ms 
#
# creates a C++ wrapper to Pentium optimized assembly functions
# for (possibly) one-way hashing
#
# This softare was written by Leonard Janke (janke@unixg.ubc.ca)
# in 1996-7 and is entered, by him, into the public domain.

!include ..\config.ms

OBJS=chnhash.obj sha.obj sha0.obj sha1.obj \
     md4ish.obj md5.obj rmd128.obj rmd160.obj \
     md5cp.obj sha0cp.obj sha1cp.obj rmd128cp.obj rmd160cp.obj
INCLUDEDIR=/I..
LIBDIR=..\lib
NAME=ch
MISCHEADERS=chmisc.hpp chmisc.inl chtypes.h

$(NAME).lib :	$(OBJS)
	$(LIBR) $(LIBQUIET) $(LIBNAME)$(NAME).lib $(LIBUPDATE) $(OBJS) 

# compression functions

.c.obj : 
	$(CC) $(CCQUIET) $(CCFLAGS) $(INCLUDEDIR) $*.c

md5cp.obj : md5cp.c

sha0cp.obj : sha0cp.c

sha1cp.obj : sha1cp.c

rmd128cp.obj : rmd128cp.c

rmd160cp.obj : rmd160cp.c

#
# abstract base classes
#

chnhash.obj :     chnhash.hpp chnhash.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

md4ish.obj :     $(MISCHEADERS) chnhash.hpp md4ish.hpp md4ish.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

#
# MD5 stuff
#

md5.obj :	$(MISCHEADERS) chnhash.hpp md4ish.hpp md5.hpp md5.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

#
# SHA stuff
#

sha.obj :	$(MISCHEADERS) chnhash.hpp sha.hpp sha.cpp 
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

sha1.obj :	$(MISCHEADERS) chnhash.hpp sha.hpp sha1.hpp sha1.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

sha0.obj :	$(MISCHEADERS) chnhash.hpp sha.hpp sha0.hpp sha0.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

#
# RIPE-MD128 stuff
#

rmd128.obj :	$(MISCHEADERS) chnhash.hpp rmd128.hpp rmd128.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

#
# RIPE-MD160 stuff
#

rmd160.obj :	$(MISCHEADERS) chnhash.hpp rmd160.hpp rmd160.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDEDIR) $*.cpp

#
# installation stuff
#

install-lib :	$(NAME).lib
	$(COPY) $(NAME).lib $(LIBDIR)\$(NAME).lib 

install-headers :
	$(COPY) *.hpp ..\include\chnhash
	$(COPY) *.h ..\include\chnhash
	$(COPY) *.inl ..\include\chnhash

install :	install-lib install-headers

#
# clean up stuff
#

clean :
	if exist *~ $(DELETE) *~
	if exist *.obj $(DELETE) *.obj

distclean :	clean
	if exist *cp.c $(DELETE) *cp.c
	if exist $(NAME).lib $(DELETE) $(NAME).lib
	if exist clink.h $(DELETE) clink.h
