#
# chnhash/makefile.wat 
#
# creates a C++ wrapper to Pentium optimized assembly functions
# for (possibly) one-way hashing
#
# This softare was written by Leonard Janke (janke@unixg.ubc.ca)
# in 1996-7 and is entered, by him, into the public domain.

!include ..\config.wat

OBJS=chnhash.obj sha.obj sha0.obj sha1.obj &
     md4ish.obj md5.obj rmd128.obj rmd160.obj &
     md5cp.obj sha0cp.obj sha1cp.obj rmd128cp.obj rmd160cp.obj
INCLUDE=/i..\
LIBDIR=..\lib
NAME=ch
MISCHEADERS=chmisc.hpp chmisc.inl chtypes.h clink.h

$(NAME).lib :	$(OBJS)
	for %o in ($?) do $(LIB) $(LIBQUIET) $(LIBNAME) $(NAME).lib &
        $(LIBUPDATE) %o

# compression functions

.asm.obj : 
	$(ASSEMBLER) $(ASSEMBLEQUIET) $[.

md5cp.obj : md5cp.asm

sha0cp.obj : sha0cp.asm

sha1cp.obj : sha1cp.asm

rmd128cp.obj : rmd128cp.asm

rmd160cp.obj : rmd160cp.asm

#
# abstract base classes
#

chnhash.obj :     chnhash.hpp chnhash.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDE) $].

md4ish.obj :     $(MISCHEADERS) chnhash.hpp md4ish.hpp md4ish.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDE) $].

#
# MD5 stuff
#

md5.obj :	$(MISCHEADERS) chnhash.hpp md4ish.hpp md5.hpp md5.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $].

#
# SHA stuff
#

sha.obj :	$(MISCHEADERS) chnhash.hpp sha.hpp sha.cpp 
	$(CPP) $(CPPQUIET) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDE) $].

sha1.obj :	$(MISCHEADERS) chnhash.hpp sha.hpp sha1.hpp sha1.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $].

sha0.obj :	$(MISCHEADERS) chnhash.hpp sha.hpp sha0.hpp sha0.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $].

#
# RIPE-MD128 stuff
#

rmd128.obj :	$(MISCHEADERS) chnhash.hpp rmd128.hpp rmd128.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS)  $(CPPOPTIMIZE) $(INCLUDE) $].

#
# RIPE-MD160 stuff
#

rmd160.obj :	$(MISCHEADERS) chnhash.hpp rmd160.hpp rmd160.cpp
	$(CPP) $(CPPQUIET) $(CPPFLAGS) $(CPPOPTIMIZE) $(INCLUDE) $].

#
# installation stuff
#

install-lib :	$(NAME).lib .SYMBOLIC
	copy $(NAME).lib $(LIBDIR)\$(NAME).lib 

install-headers
	copy *.hpp ..\include\chnhash\
	copy *.h ..\include\chnhash\
	copy *.inl ..\include\chnhash\

install :	install-lib install-headers

#
# clean up stuff
#

clean
	@if exist *~ $(DELETE) *~
	@if exist *.obj $(DELETE) *.obj

distclean :	clean .SYMBOLIC
	@if exist *.asm $(DELETE) *.asm
	@if exist $(NAME).lib $(DELETE) $(NAME).lib
	@if exist $(NAME).bak $(DELETE) $(NAME).bak
	@if exist $(NAME).lst $(DELETE) $(NAME).lst
	@if exist clink.h $(DELETE) clink.h
